@@ -39,12 +39,16 @@ class RenderPartialWithRecordIdentificationController < ActionController::Base
     @developers = Developer.find(:all)
     render :partial => @developers
   end
+
+  def render_with_record_collection_and_spacer_template
+    @developer = Developer.find(1)
+    render :partial => @developer.projects, :spacer_template => 'test/partial_only'
+  end
 end
-RenderPartialWithRecordIdentificationController.view_paths = [ File.dirname(__FILE__) + "/../fixtures/" ]
 
 class RenderPartialWithRecordIdentificationTest < ActiveRecordTestCase
   fixtures :developers, :projects, :developers_projects, :topics, :replies, :companies, :mascots
-  
+
   def setup
     @controller = RenderPartialWithRecordIdentificationController.new
     @request    = ActionController::TestRequest.new
@@ -55,26 +59,36 @@ class RenderPartialWithRecordIdentificationTest < ActiveRecordTestCase
   def test_rendering_partial_with_has_many_and_belongs_to_association
     get :render_with_has_many_and_belongs_to_association
     assert_template 'projects/_project'
+    assert_equal 'Active RecordActive Controller', @response.body
   end
-  
+
   def test_rendering_partial_with_has_many_association
     get :render_with_has_many_association
     assert_template 'replies/_reply'
+    assert_equal 'Birdman is better!', @response.body
   end
-  
+
   def test_rendering_partial_with_named_scope
     get :render_with_named_scope
     assert_template 'replies/_reply'
+    assert_equal 'Birdman is better!Nuh uh!', @response.body
   end
-  
+
   def test_render_with_record
     get :render_with_record
     assert_template 'developers/_developer'
+    assert_equal 'David', @response.body
   end
-  
+
   def test_render_with_record_collection
     get :render_with_record_collection
     assert_template 'developers/_developer'
+    assert_equal 'DavidJamisfixture_3fixture_4fixture_5fixture_6fixture_7fixture_8fixture_9fixture_10Jamis', @response.body
+  end
+
+  def test_render_with_record_collection_and_spacer_template
+    get :render_with_record_collection_and_spacer_template
+    assert_equal 'Active Recordonly partialActive Controller', @response.body
   end
 
   def test_rendering_partial_with_has_one_association
@@ -116,7 +130,6 @@ class RenderPartialWithRecordIdentificationController < ActionController::Base
     render :partial => @developers
   end
 end
-RenderPartialWithRecordIdentificationController.view_paths = [ File.dirname(__FILE__) + "/../fixtures/" ]
 
 class Game < Struct.new(:name, :id)
   def to_param
@@ -134,7 +147,6 @@ module Fun
       render :partial => [ Game.new("Pong"), Game.new("Tank") ]
     end
   end
-  NestedController.view_paths = [ File.dirname(__FILE__) + "/../fixtures/" ]
 
   module Serious
     class NestedDeeperController < ActionController::Base
@@ -146,7 +158,6 @@ module Fun
         render :partial => [ Game.new("Chess"), Game.new("Sudoku"), Game.new("Solitaire") ]
       end
     end
-    NestedDeeperController.view_paths = [ File.dirname(__FILE__) + "/../fixtures/" ]
   end
 end
 
@@ -161,13 +172,14 @@ class RenderPartialWithRecordIdentificationAndNestedControllersTest < ActiveReco
   def test_render_with_record_in_nested_controller
     get :render_with_record_in_nested_controller
     assert_template 'fun/games/_game'
+    assert_equal 'Pong', @response.body
   end
 
   def test_render_with_record_collection_in_nested_controller
     get :render_with_record_collection_in_nested_controller
     assert_template 'fun/games/_game'
+    assert_equal 'PongTank', @response.body
   end
-
 end
 
 class RenderPartialWithRecordIdentificationAndNestedDeeperControllersTest < ActiveRecordTestCase
@@ -181,11 +193,12 @@ class RenderPartialWithRecordIdentificationAndNestedDeeperControllersTest < Acti
   def test_render_with_record_in_deeper_nested_controller
     get :render_with_record_in_deeper_nested_controller
     assert_template 'fun/serious/games/_game'
+    assert_equal 'Chess', @response.body
   end
 
   def test_render_with_record_collection_in_deeper_nested_controller
     get :render_with_record_collection_in_deeper_nested_controller
     assert_template 'fun/serious/games/_game'
+    assert_equal 'ChessSudokuSolitaire', @response.body
   end
-
-end
\ No newline at end of file
+end
