@@ -1,7 +1,6 @@
-require File.dirname(__FILE__) + '/../abstract_unit'
+require 'abstract_unit'
 
 class Address
-
   def Address.count(conditions = nil, join = nil)
     nil
   end
@@ -20,8 +19,6 @@ class AddressesTestController < ActionController::Base
   def self.controller_path; "addresses"; end
 end
 
-AddressesTestController.view_paths = [ File.dirname(__FILE__) + "/../fixtures/" ]
-
 class AddressesTest < Test::Unit::TestCase
   def setup
     @controller = AddressesTestController.new
