@@ -1,4 +1,4 @@
-require File.dirname(__FILE__) + '/../abstract_unit'
+require 'abstract_unit'
 
 class WorkshopsController < ActionController::Base
 end
@@ -9,11 +9,11 @@ class Workshop
   def initialize(id, new_record)
     @id, @new_record = id, new_record
   end
-  
+
   def new_record?
     @new_record
   end
-  
+
   def to_s
     id.to_s
   end
@@ -24,32 +24,32 @@ class RedirectController < ActionController::Base
     redirect_to :action => "hello_world"
   end
 
-  def redirect_with_status 
+  def redirect_with_status
     redirect_to({:action => "hello_world", :status => 301})
-  end 
+  end
 
   def redirect_with_status_hash
     redirect_to({:action => "hello_world"}, {:status => 301})
-  end 
+  end
 
-  def url_redirect_with_status 
+  def url_redirect_with_status
     redirect_to("http://www.example.com", :status => :moved_permanently)
-  end 
-   
-  def url_redirect_with_status_hash 
-    redirect_to("http://www.example.com", {:status => 301})
-  end 
+  end
 
-  def relative_url_redirect_with_status 
+  def url_redirect_with_status_hash
+    redirect_to("http://www.example.com", {:status => 301})
+  end
+
+  def relative_url_redirect_with_status
     redirect_to("/things/stuff", :status => :found)
-  end 
-   
+  end
+
   def relative_url_redirect_with_status_hash
     redirect_to("/things/stuff", {:status => 301})
-  end 
-   
-  def redirect_to_back_with_status 
-    redirect_to :back, :status => 307 
+  end
+
+  def redirect_to_back_with_status
+    redirect_to :back, :status => 307
   end
 
   def host_redirect
@@ -65,6 +65,18 @@ class RedirectController < ActionController::Base
     redirect_to :action => "hello_world"
   end
 
+  def redirect_to_url
+    redirect_to "http://www.rubyonrails.org/"
+  end
+
+  def redirect_to_url_with_unescaped_query_string
+    redirect_to "http://dev.rubyonrails.org/query?status=new"
+  end
+
+  def redirect_to_url_with_complex_scheme
+    redirect_to "x-test+scheme.complex:redirect"
+  end
+
   def redirect_to_back
     redirect_to :back
   end
@@ -77,10 +89,14 @@ class RedirectController < ActionController::Base
     redirect_to Workshop.new(5, true)
   end
 
+  def redirect_to_nil
+    redirect_to nil
+  end
+
   def rescue_errors(e) raise e end
-    
+
   def rescue_action(e) raise end
-  
+
   protected
     def dashbord_url(id, message)
       url_for :action => "dashboard", :params => { "id" => id, "message" => message }
@@ -106,48 +122,48 @@ class RedirectTest < Test::Unit::TestCase
     assert_equal "http://test.host/redirect/hello_world", redirect_to_url
   end
 
-  def test_redirect_with_status 
-    get :redirect_with_status 
-    assert_response 301 
-    assert_equal "http://test.host/redirect/hello_world", redirect_to_url 
-  end 
+  def test_redirect_with_status
+    get :redirect_with_status
+    assert_response 301
+    assert_equal "http://test.host/redirect/hello_world", redirect_to_url
+  end
 
-  def test_redirect_with_status_hash 
+  def test_redirect_with_status_hash
     get :redirect_with_status_hash
-    assert_response 301 
-    assert_equal "http://test.host/redirect/hello_world", redirect_to_url 
+    assert_response 301
+    assert_equal "http://test.host/redirect/hello_world", redirect_to_url
+  end
+
+  def test_url_redirect_with_status
+    get :url_redirect_with_status
+    assert_response 301
+    assert_equal "http://www.example.com", redirect_to_url
   end
-   
-  def test_url_redirect_with_status 
-    get :url_redirect_with_status 
-    assert_response 301 
-    assert_equal "http://www.example.com", redirect_to_url 
-  end 
 
   def test_url_redirect_with_status_hash
     get :url_redirect_with_status_hash
-    assert_response 301 
-    assert_equal "http://www.example.com", redirect_to_url 
-  end 
+    assert_response 301
+    assert_equal "http://www.example.com", redirect_to_url
+  end
 
-  
-  def test_relative_url_redirect_with_status 
-    get :relative_url_redirect_with_status 
+
+  def test_relative_url_redirect_with_status
+    get :relative_url_redirect_with_status
     assert_response 302
-    assert_equal "http://test.host/things/stuff", redirect_to_url 
-  end 
-   
+    assert_equal "http://test.host/things/stuff", redirect_to_url
+  end
+
   def test_relative_url_redirect_with_status_hash
     get :relative_url_redirect_with_status_hash
-    assert_response 301 
-    assert_equal "http://test.host/things/stuff", redirect_to_url 
-  end   
-   
-  def test_redirect_to_back_with_status 
-    @request.env["HTTP_REFERER"] = "http://www.example.com/coming/from" 
-    get :redirect_to_back_with_status 
-    assert_response 307 
-    assert_equal "http://www.example.com/coming/from", redirect_to_url 
+    assert_response 301
+    assert_equal "http://test.host/things/stuff", redirect_to_url
+  end
+
+  def test_redirect_to_back_with_status
+    @request.env["HTTP_REFERER"] = "http://www.example.com/coming/from"
+    get :redirect_to_back_with_status
+    assert_response 307
+    assert_equal "http://www.example.com/coming/from", redirect_to_url
   end
 
   def test_simple_redirect_using_options
@@ -156,21 +172,6 @@ class RedirectTest < Test::Unit::TestCase
     assert_redirected_to :action => "other_host", :only_path => false, :host => 'other.test.host'
   end
 
-  def test_redirect_error_with_pretty_diff
-    get :host_redirect
-    assert_response :redirect
-    begin
-      assert_redirected_to :action => "other_host", :only_path => true
-    rescue Test::Unit::AssertionFailedError => err
-      expected_msg, redirection_msg, diff_msg = err.message.scan(/<\{[^\}]+\}>/).collect { |s| s[2..-3] }
-      assert_match %r("only_path"=>false),        redirection_msg
-      assert_match %r("host"=>"other.test.host"), redirection_msg
-      assert_match %r("action"=>"other_host"),    redirection_msg
-      assert_match %r("only_path"=>false),        diff_msg
-      assert_match %r("host"=>"other.test.host"), diff_msg
-    end
-  end
-
   def test_module_redirect
     get :module_redirect
     assert_response :redirect
@@ -189,31 +190,62 @@ class RedirectTest < Test::Unit::TestCase
     assert_equal "world", assigns["hello"]
   end
 
+  def test_redirect_to_url
+    get :redirect_to_url
+    assert_response :redirect
+    assert_redirected_to "http://www.rubyonrails.org/"
+  end
+
+  def test_redirect_to_url_with_unescaped_query_string
+    get :redirect_to_url_with_unescaped_query_string
+    assert_response :redirect
+    assert_redirected_to "http://dev.rubyonrails.org/query?status=new"
+  end
+
+  def test_redirect_to_url_with_complex_scheme
+    get :redirect_to_url_with_complex_scheme
+    assert_response :redirect
+    assert_equal "x-test+scheme.complex:redirect", redirect_to_url
+  end
+
   def test_redirect_to_back
     @request.env["HTTP_REFERER"] = "http://www.example.com/coming/from"
     get :redirect_to_back
     assert_response :redirect
     assert_equal "http://www.example.com/coming/from", redirect_to_url
   end
-  
+
   def test_redirect_to_back_with_no_referer
     assert_raises(ActionController::RedirectBackError) {
       @request.env["HTTP_REFERER"] = nil
       get :redirect_to_back
     }
   end
-  
+
   def test_redirect_to_record
     ActionController::Routing::Routes.draw do |map|
       map.resources :workshops
       map.connect ':controller/:action/:id'
     end
-    
+
     get :redirect_to_existing_record
     assert_equal "http://test.host/workshops/5", redirect_to_url
+    assert_redirected_to Workshop.new(5, false)
 
     get :redirect_to_new_record
     assert_equal "http://test.host/workshops", redirect_to_url
+    assert_redirected_to Workshop.new(5, true)
+  end
+
+  def test_redirect_with_partial_params
+    get :module_redirect
+    assert_redirected_to :action => 'hello_world'
+  end
+
+  def test_redirect_to_nil
+    assert_raises(ActionController::ActionControllerError) do
+      get :redirect_to_nil
+    end
   end
 end
 
@@ -252,7 +284,7 @@ module ModuleTest
     def test_module_redirect_using_options
       get :module_redirect
       assert_response :redirect
-      assert_redirected_to :controller => 'redirect', :action => "hello_world"
+      assert_redirected_to :controller => '/redirect', :action => "hello_world"
     end
   end
 end
