@@ -1,20 +1,12 @@
-require "#{File.dirname(__FILE__)}/../abstract_unit"
+require 'abstract_unit'
 
-class JavaScriptHelperTest < Test::Unit::TestCase
-  include ActionView::Helpers::JavaScriptHelper
+class JavaScriptHelperTest < ActionView::TestCase
+  tests ActionView::Helpers::JavaScriptHelper
 
-  include ActionView::Helpers::UrlHelper
-  include ActionView::Helpers::TagHelper
-  include ActionView::Helpers::TextHelper
-  include ActionView::Helpers::FormHelper
-  include ActionView::Helpers::CaptureHelper
+  attr_accessor :template_format, :output_buffer
 
-  def test_define_javascript_functions
-    # check if prototype.js is included first
-    assert_not_nil define_javascript_functions.split("\n")[1].match(/Prototype JavaScript framework/)
-
-    # check that scriptaculous.js is not in here, only needed if loaded remotely
-    assert_nil define_javascript_functions.split("\n")[1].match(/var Scriptaculous = \{/)
+  def setup
+    @template = self
   end
 
   def test_escape_javascript
@@ -54,7 +46,7 @@ class JavaScriptHelperTest < Test::Unit::TestCase
   end
 
   def test_link_to_function_with_href
-    assert_dom_equal %(<a href="http://example.com/" onclick="alert('Hello world!'); return false;">Greeting</a>), 
+    assert_dom_equal %(<a href="http://example.com/" onclick="alert('Hello world!'); return false;">Greeting</a>),
       link_to_function("Greeting", "alert('Hello world!')", :href => 'http://example.com/')
   end
 
@@ -88,8 +80,12 @@ class JavaScriptHelperTest < Test::Unit::TestCase
   end
 
   def test_javascript_tag
+    self.output_buffer = 'foo'
+
     assert_dom_equal "<script type=\"text/javascript\">\n//<![CDATA[\nalert('hello')\n//]]>\n</script>",
       javascript_tag("alert('hello')")
+
+    assert_equal 'foo', output_buffer, 'javascript_tag without a block should not concat to output_buffer'
   end
 
   def test_javascript_tag_with_options
@@ -97,16 +93,16 @@ class JavaScriptHelperTest < Test::Unit::TestCase
       javascript_tag("alert('hello')", :id => "the_js_tag")
   end
 
-  def test_javascript_tag_with_block
-    _erbout = ''
-    javascript_tag { _erbout.concat "alert('hello')" }
-    assert_dom_equal "<script type=\"text/javascript\">\n//<![CDATA[\nalert('hello')\n//]]>\n</script>", _erbout
+  def test_javascript_tag_with_block_in_erb
+    __in_erb_template = ''
+    javascript_tag { concat "alert('hello')" }
+    assert_dom_equal "<script type=\"text/javascript\">\n//<![CDATA[\nalert('hello')\n//]]>\n</script>", output_buffer
   end
 
-  def test_javascript_tag_with_block_and_options
-    _erbout = ''
-    javascript_tag(:id => "the_js_tag") { _erbout.concat "alert('hello')" }
-    assert_dom_equal "<script id=\"the_js_tag\" type=\"text/javascript\">\n//<![CDATA[\nalert('hello')\n//]]>\n</script>", _erbout
+  def test_javascript_tag_with_block_and_options_in_erb
+    __in_erb_template = ''
+    javascript_tag(:id => "the_js_tag") { concat "alert('hello')" }
+    assert_dom_equal "<script id=\"the_js_tag\" type=\"text/javascript\">\n//<![CDATA[\nalert('hello')\n//]]>\n</script>", output_buffer
   end
 
   def test_javascript_cdata_section
